import {RouteProp} from '@react-navigation/native';
import React, {useEffect, useRef, useState} from 'react';
import {StatusBar, StyleSheet} from 'react-native';
import {SafeAreaView} from 'react-native-safe-area-context';
import {RootStackParamList} from '../App';
import ButtonLayout from '../components/ButtonLayout';
import {Layout} from '../types/ButtonLayout';

interface Props {
  remoteAddress: string;
  route: RouteProp<RootStackParamList, 'Deck'>;
}

function requestLayout(ws: WebSocket) {
  ws.send(
    JSON.stringify({
      messageType: 'layout',
    }),
  );
}

export default function Deck(props: Props) {
  const remoteAddress = props.route.params.remoteAddress;

  const [layout, setLayout] = useState<Layout>({buttons: []});

  const ws = useRef<WebSocket | null>(null);
  useEffect(() => {
    // TODO check if remote address is valid
    ws.current = new WebSocket(`ws://${remoteAddress}:8069/ws`);

    ws.current.onopen = () => {
      requestLayout(ws.current!);
    };
    ws.current.onerror = (e) => {
      // TODO tell the user something is wrong
      console.error(e);
    };

    ws.current.onmessage = (msg) => {
      // TODO also add a mesasge type to response messages
      const payload = JSON.parse(msg.data);
      if (payload.buttons) {
        setLayout(payload);
      }
    };

    return () => {
      ws.current?.close();
    };
  }, [remoteAddress]);

  function buttonPress(index: number) {
    ws.current?.send(
      JSON.stringify({
        messageType: 'buttonPress',
        buttonIndex: index,
        long: false,
      }),
    );
  }

  return (
    <>
      <StatusBar hidden={true} />
      <SafeAreaView style={style.view}>
        <ButtonLayout layout={layout} onButtonPress={buttonPress} />
      </SafeAreaView>
    </>
  );
}

const style = StyleSheet.create({
  view: {
    flex: 1,
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
});
